//    Copyright (C) 2019, ETH Zurich, D-ITET, Paul Beuchat
//
//    This file is part of D-FaLL-System.
//    
//    D-FaLL-System is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//    
//    D-FaLL-System is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//    
//    You should have received a copy of the GNU General Public License
//    along with D-FaLL-System.  If not, see <http://www.gnu.org/licenses/>.
//    
//
//    ----------------------------------------------------------------------------------
//    DDDD        FFFFF        L     L           SSSS  Y   Y   SSSS  TTTTT  EEEEE  M   M
//    D   D       F      aaa   L     L          S       Y Y   S        T    E      MM MM
//    D   D  ---  FFFF  a   a  L     L     ---   SSS     Y     SSS     T    EEE    M M M
//    D   D       F     a  aa  L     L              S    Y        S    T    E      M   M
//    DDDD        F      aa a  LLLL  LLLL       SSSS     Y    SSSS     T    EEEEE  M   M
//
//
//    DESCRIPTION:
//    The service that provides data to the web interface
//
//    ----------------------------------------------------------------------------------





// INCLUDE THE HEADER
#include "nodes/AgentStatusForWebInterface.h"











//    ----------------------------------------------------------------------------------
//    FFFFF  U   U  N   N   CCCC  TTTTT  III   OOO   N   N
//    F      U   U  NN  N  C        T     I   O   O  NN  N
//    FFF    U   U  N N N  C        T     I   O   O  N N N
//    F      U   U  N  NN  C        T     I   O   O  N  NN
//    F       UUU   N   N   CCCC    T    III   OOO   N   N
//
//    III M   M PPPP  L     EEEEE M   M EEEEE N   N TTTTT   A   TTTTT III  OOO  N   N
//     I  MM MM P   P L     E     MM MM E     NN  N   T    A A    T    I  O   O NN  N
//     I  M M M PPPP  L     EEE   M M M EEE   N N N   T   A   A   T    I  O   O N N N
//     I  M   M P     L     E     M   M E     N  NN   T   AAAAA   T    I  O   O N  NN
//    III M   M P     LLLLL EEEEE M   M EEEEE N   N   T   A   A   T   III  OOO  N   N
//    ----------------------------------------------------------------------------------



// CALLBACKS FOR MESSAGES WITH THE STATUS OF THINGS

// > For the status of the crazyradio
void crazyRadioStatusCallback(const std_msgs::Int32& msg)
{
	m_crazyradio_status = msg.data;
}


// > For the flying state of the agent
void agentOperatingStateCallback(const std_msgs::Int32& msg)
{
	m_agent_operating_state = msg.data;
}

// > For the instant controller
void instantControllerChangedCallback(const std_msgs::Int32& msg)
{
	m_instant_controller = msg.data;
}

// > For the battery level
void newBatteryLevelCallback(const std_msgs::Int32& msg)
{
	m_battery_level = msg.data;
}

// > For the Default Controller Setpoint
void defaultControllerSetpointChangedCallback(const SetpointWithHeader& newSetpoint)
{
    m_setpoint_default[0] = newSetpoint.x;
    m_setpoint_default[1] = newSetpoint.y;
    m_setpoint_default[2] = newSetpoint.z;
    m_setpoint_default[3] = newSetpoint.yaw;
}

// > For the Student Controller Setpoint
void studentControllerSetpointChangedCallback(const SetpointWithHeader& newSetpoint)
{
    m_setpoint_student[0] = newSetpoint.x;
    m_setpoint_student[1] = newSetpoint.y;
    m_setpoint_student[2] = newSetpoint.z;
    m_setpoint_student[3] = newSetpoint.yaw;
}





// SERVICE CALLBACK FOR PROVIDING STATUS TO THE WEB INTERFACE
bool statusForWebInterfaceCallback(IntStringService::Request &request, IntStringService::Response &response)
{
	// Concatenate the json together using a string stream
	std::stringstream ss;
	ss << "{\"crazyradiostatus\": \"" << m_crazyradio_status << "\"}";
	std::string s = ss.str();
	// Put the string into the response
	//response.data = "test of service for web interface";
	//response.data = "{\"crazyradiostatus\": \"%d\"}", m_crazyradio_status;
	response.data = ss.str();

	// Return success
	return true;
}




//    ----------------------------------------------------------------------------------
//    M   M    A    III  N   N
//    MM MM   A A    I   NN  N
//    M M M  A   A   I   N N N
//    M   M  AAAAA   I   N  NN
//    M   M  A   A  III  N   N
//    ----------------------------------------------------------------------------------

int main(int argc, char* argv[])
{
	// Starting the ROS-node
	ros::init(argc, argv, "AgentStatusForWebInterface");

	// Create a "ros::NodeHandle" type local variable "nodeHandle"
	// as the current node, the "~" indcates that "self" is the
	// node handle assigned to this variable.
	ros::NodeHandle nodeHandle("~");

	// Get the namespace of this "AgentStatusForWebInterface" node
	std::string m_namespace = ros::this_node::getNamespace();
	ROS_INFO_STREAM("[AGENT STATUS FOR WEB INTERFACE] ros::this_node::getNamespace() =  " << m_namespace);





	// SUBSCRIBERS

	// FOR THE CRAZY RADIO STATUS:
	// > Get a node handle to the Crazy Radio
	std::string namespace_to_crazyradio = m_namespace + "/CrazyRadio";
	ros::NodeHandle nodeHandle_to_crazyradio(namespace_to_crazyradio);
	// > Subscribe to the topic
	ros::Subscriber crazyRadioStatusSubscriber = nodeHandle_to_crazyradio.subscribe("CrazyRadioStatus", 1, crazyRadioStatusCallback);

	// FOR THE FLYING STATE OF THE AGENT
	// > Get a node handle to the Flying Agent Client
	std::string namespace_to_FlyingAgentClient = m_namespace + "/FlyingAgentClient";
	ros::NodeHandle nodeHandle_to_FlyingAgentClient(namespace_to_FlyingAgentClient);
	// > Subscribe to the topic
	ros::Subscriber agentOperatingStateSubscriber = nodeHandle_to_FlyingAgentClient.subscribe("FlyingState", 1, agentOperatingStateCallback);

	// FOR THE INSTANT CONTROLLER
	// > Subscribe to the topic
	ros::Subscriber instantControllerSubscriber = nodeHandle_to_FlyingAgentClient.subscribe("ControllerUsed", 1, instantControllerChangedCallback);

	// FOR THE BATTERY LEVEL
	// > Get a node handle to the Battery Monitor
	std::string namespace_to_BatteryMonitor = m_namespace + "/BatteryMonitor";
	ros::NodeHandle nodeHandle_to_BatteryMonitor(namespace_to_BatteryMonitor);
	// > Subscribe to the topic
	ros::Subscriber newBatteryLevelSubscriber = nodeHandle_to_BatteryMonitor.subscribe("Level", 1, newBatteryLevelCallback);

	// FOR THE DEFAULT CONTROLLER SETPOINT
	// > Get a node handle to the Default Controller Service
	std::string namespace_to_DefaultController = m_namespace + "/DefaultControllerService";
	ros::NodeHandle nodeHandle_to_DefaultController(namespace_to_DefaultController);
	// > Subscribe to the topic
	ros::Subscriber defaultControllerSetpointChangedSubscriber = nodeHandle_to_DefaultController.subscribe("SetpointChanged", 1, defaultControllerSetpointChangedCallback);

	// FOR THE STUDENT CONTROLLER SETPOINT
	// > Get a node handle to the Student Controller Service
	std::string namespace_to_StudentController = m_namespace + "/StudentControllerService";
	ros::NodeHandle nodeHandle_to_StudentController(namespace_to_StudentController);
	// > Subscribe to the topic
	ros::Subscriber stuentControllerSetpointChangedSubscriber = nodeHandle_to_StudentController.subscribe("SetpointChanged", 1, studentControllerSetpointChangedCallback);





	// SERVICE SERVERS

	// Service callback for provide status to the Web Interface
	ros::ServiceServer statusForWebInterfaceService = nodeHandle.advertiseService("StatusAsJson", statusForWebInterfaceCallback);





	// Inform the user the this node is ready
	ROS_INFO("[AGENT STATUS FOR WEB INTERFACE] Ready :-)");
	// Spin as a single-thread node
	ros::spin();

	return 0;
}
